use core::fmt;

use serde::Deserialize;

#[derive(Deserialize, Debug)]
pub struct MinecraftVersion {
    pub version: String,
    pub version_type: String,
    pub date: String,
    pub major: bool,
}

#[derive(Deserialize, Debug)]
pub struct Mod {
    pub title: String,
    pub description: String,
    pub project_id: String,
}

impl fmt::Display for Mod {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{}", self.title)
    }
}

#[derive(Deserialize, Debug)]
pub struct Search {
    hits: Vec<Mod>,
}

#[derive(Deserialize, Debug)]
pub struct ProjectVersion {
    pub name: String,
    pub game_versions: Vec<String>,
    pub version_type: String,
    pub id: String,
    pub project_id: String,
}

pub async fn get_minecraft_versions() -> Result<Vec<MinecraftVersion>, reqwest::Error> {
    reqwest::get("https://api.modrinth.com/v2/tag/game_version")
        .await?
        .json::<Vec<MinecraftVersion>>()
        .await
}

pub async fn search_projects(
    query: String,
    version: &String,
    loader: &String,
) -> Result<Vec<Mod>, reqwest::Error> {
    let url = format!("https://api.modrinth.com/v2/search?query={query}&facets=[[\"project_type:mod\"],[\"versions:{version}\"],[\"categories:{loader}\"]]");

    let response = reqwest::get(url).await?.json::<Search>().await?;

    Ok(response.hits)
}

pub async fn get_project_versions(
    id: &String,
    version: &String,
    loader: &String,
) -> Result<Vec<ProjectVersion>, reqwest::Error> {
    let url = format!("https://api.modrinth.com/v2/project/{id}/version?loaders=[\"{loader}\"]&game_versions=[\"{version}\"]");

    let response = reqwest::get(url)
        .await?
        .json::<Vec<ProjectVersion>>()
        .await?;

    Ok(response)
}
